import cdsapi as cd
import datetime as dt
import numpy as np
import argparse

import os

from dateutil.relativedelta import relativedelta
from netCDF4 import Dataset  # http://code.google.com/p/netcdf4-python/

import warnings
warnings.filterwarnings('ignore')

#######################
# Given year and month, retrieve list of days for that month, and return them in standard format YYYY-MM-DD
def days_of_month(y, m):
    d0 = dt.datetime(y, m, 1) # First day of month 
    d1 = d0 + relativedelta(months=1) # Add one month (lol)
    
    # Iteraltively append formatted days to a list
    out = list()
    while d0 < d1:
        out.append(d0.strftime('%Y-%m-%d'))
        d0 += dt.timedelta(days=1)
    return out
    
#######################
# checkpoint -- read days already downloaded and skips them by removing from the list the corresponding entry. It checks whether a file for a given date exists and removes the date from the list
def read_checkpoint(days,root_fold_data='data/', verbose=False):
    # check in folder if a day with proper formatting exists
    # Iteratively remove from the list one element and return final list, 
    # or retrieve the last date stored locally and start from there (assumes ordered downloads)
    n_days = days.copy()
    for ddate in days:
        # filter: 
        if os.path.isfile(f'{root_fold_data}f_date_{ddate}.nc'):
            if verbose:
                print(f'Skipping {ddate}')
            n_days.remove(ddate)
    return n_days

#######################
# Read API key information from a local file, which is gitignored so at least not shared across repo members and forces everyone to get own key
def read_local_key(file='../.api_key'):
    with open(file,'r',encoding='utf-8') as keyfile:
        line = keyfile.readline()
        while line:
            st_lin = line.strip()
            st_lin = st_lin.split(sep='=')
            if st_lin[0] == 'UUID':
                uuid = st_lin[1]
            if st_lin[0] == 'key':
                key = st_lin[1]
            line = keyfile.readline()
    return uuid, key

#######################
def download_ecmwf(UUID, key, download_folder, days, time_of_day, variables, verbose=True):
    
    # Create folder if not present 
    if not os.path.isdir(download_folder): 
        os.mkdir(download_folder)
        
    # From key in correct formatting
    api_key = UUID + ':' + key
    
    # checkpoint:
    days = read_checkpoint(days,root_fold_data=download_folder)
    
    # Call API And download
    c = cd.Client(key=api_key, url='https://cds.climate.copernicus.eu/api/v2')
    for ddate in days:
        if verbose:
            print(f'Downloaded request for {ddate}')
            
        # API call
        c.retrieve(
            'reanalysis-era5-single-levels',
            {
                'variable': variables,
                'product_type':'reanalysis',
                'date': ddate,
                'time': time_of_day,
                'format':'netcdf'
            },
            f'{download_folder}/f_date_{ddate}.nc')

            
